<?php

/**
 * Admin settings form.
 */
function ofed_social_sharethis_migrate_admin_form() {
  $form['ofed_social_sharethis_migrate'] = array(
    '#type' => 'fieldset',
    '#title' => t('Openfed Social Share Migrate'),
    '#collapsible' => false,
  );

  if( module_exists('sharethis')) {
    // Get sharethis config so we can show some notices if needed.
    $sharethis_config = _ofed_social_sharethis_migrate_get_sharethis_config();
    $notices = $return_value = '';

    $return_value .= "<p>". t("Openfed Social provides a tool to make it easier to switch from ShareThis.");
    $return_value .= " ". t("Note that this is an automatic procedure and may not work correctly with your ShareThis configuration.");
    $return_value .= " ". t("In order to guide you and make sure everything runs smoothly, we ran several checks on your config to warn you about potencial problems.");
    $return_value .= " ". t("If you see a \"Notices\" section below, you may have to check your installation manually after running this migration or consider a manual replacement of ShareThis instead of this automatism.");
    $return_value .= " ". t("Otherwise, if you don't see the \"Notices\" section, is very likely that the migration will run smoothly.") ."</p>";
    $return_value .= "<p>" . t("The migration will:") . '</p>';
    $return_value .= "<ul><li>" . t("Replace your ShareThis block with the one included on Openfed Social.") . '</li>';
    $return_value .= "<li>" . t("Copy your ShareThis config to Openfed Social.") . '</li>';
    $return_value .= "<li>" . t("Disable ShareThis module.") . '</li></ul><br/>';
    $form['ofed_social_sharethis_migrate']['if_sharethis'] = array(
      '#markup' => $return_value,
    );

    $notices .= '<li>' . t("You are using sharethis styles and from now on you'll start using Openfed Social default styles. You may want to adapt the styling after this migration.") . '</li>';

    // Get any warning messages if they exist.
    $notices .= _ofed_social_sharethis_migrate_detect_issues();

    if ($notices) {
      $form['ofed_social_sharethis_migrate']['notices'] = array(
        '#type' => 'fieldset',
        '#title' => t('Notices:'),
        '#collapsible' => false,
      );
      $form['ofed_social_sharethis_migrate']['notices']['list'] = array(
        '#markup' => '<p>'. t('Please take in consideration the following information:'). '</p><ul>' . $notices . '</ul>',
      );
    }

    $form['actions'] = array('#type' => 'actions');
    $form['actions']['submit'] = array('#type' => 'submit', '#value' => t('Run migration'));
  } else {
    $form['ofed_social_sharethis_migrate']['no_sharethis'] = array(
      '#markup' => t('Sharethis is disabled so there\'s no need to run a migration.<br/ >You can disable this module.'),
    );
  }

  return $form;
}

/**
 * Process migration run, updating Openfed Social configuration, based on
 * ShareThis settings.
 */
function ofed_social_sharethis_migrate_admin_form_submit($form, &$form_state) {
  $submissions = $form_state['values'];
  $sharethis_conf = _ofed_social_sharethis_migrate_get_sharethis_config();
  $sharethis_services = _ofed_social_sharethis_migrate_get_sharethis_services();
  $sharethis_services = $sharethis_services['services'];

  // Start to set Openfed Social networks based on ShareThis enabled services.
  $networks = $networks_enabled = array();
  $existing_networks_settings = ofed_social_get_options_array();
  foreach ($sharethis_services as $service) {
    // We'll add an extra variable so we can easily get the enabled networks.
    $networks_enabled[$service] = array('label' => $submissions[$service]);
  }
  $ordered_networks = $networks_enabled + $existing_networks_settings;
  foreach ($ordered_networks as $network_key => $network_value) {
    // We've to update the full network list with the current sort order.
    $networks[$network_key] = $existing_networks_settings[$network_key];

  }
  variable_set('ofed_social_networks_enabled', $networks_enabled);
  variable_set('ofed_social_networks', $networks);

  // Change ShareThis block to Openfed Social.
  // By loading sharethis block and changing only specific keys, we assure the
  // the same config.
  $block = $sharethis_conf['current_block'];
  $block['module'] = 'ofed_social';
  $block['delta'] = 'ofed_social_block';
  // Just in case, let's check if the block already exists.
  $query = db_query('SELECT bid FROM {block} WHERE module = :module AND delta = :delta AND theme = :theme', array(
    ':module' => 'ofed_social',
    ':delta' => 'ofed_social_block',
    ':theme' => variable_get('theme_default')
  ));
  $result = $query->fetchAll();
  if (count($result) == 0) {
    unset($block['bid']);
    drupal_write_record('block', $block);
  } else {
    $block['bid'] = $result[0]->bid;
    drupal_write_record('block', $block, array('module','delta','theme'));
  }

  // Set the custom template in place
  switch ($sharethis_conf['buttons']) {
    case 'stbc_large':
      variable_set('ofed_social_theme_links', 'large');
      break;
    case 'stbc_':
      variable_set('ofed_social_theme_links', 'small');
      break;
    case 'stbc_custom':
      variable_set('ofed_social_theme_links', 'custom');
      break;
    default:
      variable_set('ofed_social_theme_links', 'default');
      break;
  }
  // We'll set Sharethis as the theme to be used, since the user is migrating...
  variable_get('ofed_social_use_sharethis_theme', 1);

  // Final step is disabling sharethis since we don't need it anymore.
  module_disable(array('sharethis'));
  drupal_uninstall_modules(array('sharethis'));

  // Now clearing the cache and sending a message.
  cache_clear_all();
  drupal_set_message(t('Migration ran successfully. You may now disable Openfed Social ShareThis Migrate.'));
  return;
}

/**
 * Helper function to get the current sharethis configurations.
 *
 * @return array with current sharethis config.
 */
function _ofed_social_sharethis_migrate_get_sharethis_config() {
  $current_options_array = sharethis_get_options_array();
  return array(
    'buttons' => $current_options_array['buttons'],
    'services' => $current_options_array['services'],
    'option_extras' => $current_options_array['option_extras'],
    'sharethis_location' => variable_get('sharethis_location', 'content'),
    'current_block' => _ofed_social_sharethis_migrate_block_load('sharethis', 'sharethis_block'),
  );
}

/**
 * Helper function to get sharethis enabled services as an array.
 *
 * @return array with sharethis enabled services and a warning message if any
 * service is discarded.
 */
function _ofed_social_sharethis_migrate_get_sharethis_services() {
  // Get Openfed Social current config, just for filtering purposes.
  $networks = ofed_social_get_options_array();

  // Sharethis stores services as a string, divided by a comma.
  $sharethis_config = _ofed_social_sharethis_migrate_get_sharethis_config();
  $services = explode(',', $sharethis_config['services']);
  $sharethis_services = array();
  foreach ($services as $service) {
    // This will get the service key and label.
    $service = explode(':', str_replace('"', '', $service));
    if (isset($networks[$service[1]])) {
      $sharethis_services['services'][$service[1]] = $service[1];
    } else {
      $sharethis_services['message'] .= '<li>' . $service[0] . '</li>';
    }
  }

  return $sharethis_services;
}

/**
 * Helper function to detect possible problems.
 *
 * @return string a message string with all the warnings.
 */
function _ofed_social_sharethis_migrate_detect_issues() {
  $message = '';
  $sharethis_config = _ofed_social_sharethis_migrate_get_sharethis_config();

  // Check if there are more services enabled than the ones we're trying to
  // import. If so, a message should be displayed.
  $services = _ofed_social_sharethis_migrate_get_sharethis_services();
  if (isset($services['message'])) {
    $message .= '<li>' . t('You\'re currently using more services than the ones available on Openfed Social:') . '</li>';
    $message .= '<ul>'. $services['message'] .'</ul>';
  }

  // Check if there're extra options enabled and if so, show a warning.
  $sharethis_config_option_extras = array_filter($sharethis_config['option_extras']);
  if (!empty($sharethis_config_option_extras)) {
    $message .= '<li>' . t('You have Sharethis extra options enabled but these can\'t be migrated. You should find an alternative.') . '</li>';
  }

  // Check if the display type is "block" because migration cannot be fully
  // completed otherwise.
  if ($sharethis_config['sharethis_location'] != 'block') {
    $message .= '<li>' . t('You are using a location other than "block" which means that you must position Openfed Social block manually after running the migration.') . '</li>';
  }

  return $message;

}
